@@ -21,6 +21,31 @@ proc CmdSwapWindows {} {
   UpdateWindow both 
 }
 
+proc BatchReceiveFTP { inst } {
+  global glob
+  if {[IsFTP $glob($inst,pwd)]} {
+    PopInfo "You can only issue a receive request to a non-FTP directory"
+    return
+  }
+  set olddir $glob([Opposite $inst],pwd)
+  foreach itemblock $glob(batchlist) {
+    set item [lindex $itemblock 0]
+    set r [regexp {ftp://([^/]*)(.*)} $item match ftpI elem]
+    if {!$r} {
+      PopWarn "Can't parse $item as FTP file"
+    } else {
+      NewPwd [Opposite $inst] ftp://$ftpI/
+      set r [Try { FTP_GetFile $ftpI "$elem" "$glob($inst,pwd)/[file tail $elem]" [lindex $itemblock 1] 0 } "" 1]
+    }
+  }
+  set glob(batchlist) {}
+  set glob(forceupdate) 1
+  NewPwd [Opposite $inst] $olddir
+  UpdateWindow both
+  set glob(forceupdate) 0
+}
+
+
 proc CmdCopy {{resume 0}} {
   global glob
   if { $glob(left,pwd) == $glob(right,pwd) } {
@@ -53,6 +78,7 @@ proc CmdCopy_ { listb_name filelist_var frompwd topwd inst resume} {
         set olddir $glob($inst,pwd)
         set r [Try { CopyFromFTPRecursive "$frompwd/[lindex $elem 1]" "$topwd" $inst $resume } "" 1]
         NewPwd $inst $olddir
+        UpdateWindow $inst
       }
       fl  -
       fn  {
@@ -75,16 +101,27 @@ proc CmdCopy_ { listb_name filelist_var frompwd topwd inst resume} {
           }
         }
       }
-      l   -
+      l   { 
+        Log "Copying $frompwd/[lindex $elem 1] to $topwd"
+        set r [regexp {ftp://([^/]*)(.*)} $topwd match ftpI directory]
+        if {$r} {
+          set glob(forceupdate) 1
+          Try { FTP_PutFile $ftpI $frompwd/[lindex $elem 1] $directory/[lindex $elem 1] [lindex $elem 3] } "" 1
+        } else {
+          if {[CheckWhoOwns $topwd/[lindex $elem 1] overwrite]} {
+            Try { exec $config(cmd,cp) $frompwd/[lindex $elem 1] $topwd } "" 1 
+          }
+        }
+      }
       n   { 
         Log "Copying $frompwd/[lindex $elem 1] to $topwd"
         set r [regexp {ftp://([^/]*)(.*)} $topwd match ftpI directory]
         if {$r} {
-          Try { FTP_PutFile $ftpI "$frompwd/[lindex $elem 1]" "$directory/[lindex $elem 1]" "[lindex $elem 3]" } "" 1
           set glob(forceupdate) 1
+          Try { FTP_PutFile $ftpI $frompwd/[lindex $elem 1] $directory/[lindex $elem 1] [lindex $elem 3] } "" 1
         } else {
           if {[CheckWhoOwns $topwd/[lindex $elem 1] overwrite]} {
-            Try { exec $config(cmd,cp) "$frompwd/[lindex $elem 1]" "$topwd" } "" 1 $glob(async)
+            Try { file copy -force -- $frompwd/[lindex $elem 1] $topwd } "" 1 
           }
         }
       }
@@ -97,8 +134,9 @@ proc CmdCopy_ { listb_name filelist_var frompwd topwd inst resume} {
         } else {
           Log "Copying $frompwd/[lindex $elem 1] to $topwd"
           if {[CheckWhoOwns $topwd/[lindex $elem 1] overwrite]} {
-            Try { cd "$frompwd"
-              exec $config(cmd,sh) -c "tar cf - \"[lindex $elem 1]\" | (cd \"$topwd\"; tar xfp - )" } "" 1 $glob(async)
+            Try { 
+              cd $frompwd
+              exec $config(cmd,sh) -c "tar cf - '[lindex $elem 1]' | (cd '$topwd'; tar xfp - )" } "" 1 $glob(async)
           }
         }
       }
@@ -109,8 +147,10 @@ proc CmdCopy_ { listb_name filelist_var frompwd topwd inst resume} {
         }
         Log "Copying $frompwd/[lindex $elem 1] to $topwd"
         if {[CheckWhoOwns $topwd/[lindex $elem 1] overwrite]} {
-          Try { cd "$frompwd/[lindex $elem 1]" ; exec $config(cmd,mkdir) "$topwd/[lindex $elem 1]";
-            exec $config(cmd,sh) -c "tar cf - . | (cd \"$topwd/[lindex $elem 1]\"; tar xfp - )" } "" 1 $glob(async)
+          Try { 
+            cd $frompwd/[lindex $elem 1]
+            file mkdir $topwd/[lindex $elem 1]
+            exec $config(cmd,sh) -c "tar cf - . | (cd '$topwd/[lindex $elem 1]'; tar xfp - )" } "" 1 $glob(async)
         }
       }
       default CantDoThat
@@ -158,7 +198,17 @@ proc CmdCopyAs_ { listb_name filelist_var frompwd topwd } {
           }
         }
       }
-      l   -
+      l   { 
+        set destfile [EntryDialog "Copy As..." "Please enter new name for destination file" $topwd/[lindex $elem 1] question]
+        if {[IsFTP $destfile]} {
+          CantDoThat
+          return
+        }
+        if {$destfile != "" } {
+          Log "Copying $frompwd/[lindex $elem 1] to $destfile"
+          Try { exec $config(cmd,cp) $frompwd/[lindex $elem 1] $destfile } "" 1  $glob(async)
+        }
+      }
       n   { 
         set destfile [EntryDialog "Copy As..." "Please enter new name for destination file" $topwd/[lindex $elem 1] question]
         if {[IsFTP $destfile]} {
@@ -167,7 +217,7 @@ proc CmdCopyAs_ { listb_name filelist_var frompwd topwd } {
         }
         if {$destfile != "" } {
           Log "Copying $frompwd/[lindex $elem 1] to $destfile"
-          Try { exec $config(cmd,cp) "$frompwd/[lindex $elem 1]" "$destfile" } "" 1  $glob(async)
+          Try { file copy -force -- $frompwd/[lindex $elem 1] $destfile } "" 1  $glob(async)
         }
       }
       d   -
@@ -179,8 +229,10 @@ proc CmdCopyAs_ { listb_name filelist_var frompwd topwd } {
         }
         if {$destdir != "" } {
           Log "Copying $frompwd/[lindex $elem 1] to $destdir"
-          Try { cd "$frompwd/[lindex $elem 1]"; exec $config(cmd,mkdir) "$destdir";
-            exec $config(cmd,sh) -c "tar cf - . | (cd \"$destdir\"; tar xfp - )" } "" 1 $glob(async)
+          Try { 
+            cd $frompwd/[lindex $elem 1]
+            file mkdir $destdir
+            exec $config(cmd,sh) -c "tar cf - . | (cd '$destdir'; tar xfp - )" } "" 1 $glob(async)
         }
       }
       default CantDoThat
@@ -297,14 +349,12 @@ proc CmdMove_ { listb_name filelist_var frompwd topwd } {
       n   { 
         if {[CheckWhoOwns $frompwd/[lindex $elem 1] move] && [CheckWhoOwns $topwd/[lindex $elem 1] overwrite]} {
           Log "Moving $frompwd/[lindex $elem 1] to $topwd"
-          Try { exec $config(cmd,mv) -f "$frompwd/[lindex $elem 1]" "$topwd" } "" 1 $glob(async)
+          Try { file rename -force -- $frompwd/[lindex $elem 1] $topwd } "" 1 $glob(async)
         }
       }
       default CantDoThat
     }
   }
-#      d   { PopInfo "Please move the directory [lindex $elem 1] by copy, then delete.\nIt's safer..."
-#          }
 }
 
 proc CmdDelete {} {
@@ -334,7 +384,7 @@ proc CmdDelete_ { listb_name filelist_var frompwd topwd inst } {
         if {$ask == 0} {
           if {[CheckWhoOwns $frompwd/[lindex $elem 1] delete]} {
             Log "Deleting $frompwd/[lindex $elem 1]"
-            Try { exec $config(cmd,rm) -f "$frompwd/[lindex $elem 1]" } "" 1
+            Try { file delete -force -- $frompwd/[lindex $elem 1] } "" 1
           }
         }
       }
@@ -346,7 +396,7 @@ proc CmdDelete_ { listb_name filelist_var frompwd topwd inst } {
         if {$ask == 0} {
           if {[CheckWhoOwns $frompwd/[lindex $elem 1] delete]} {
             Log "Deleting $frompwd/[lindex $elem 1]"
-            Try { exec $config(cmd,rm) -rf "$frompwd/[lindex $elem 1]" } "" 1 $glob(async)
+            Try { exec $config(cmd,rm) -rf $frompwd/[lindex $elem 1] } "" 1 $glob(async)
           }
         }
       }
@@ -361,8 +411,8 @@ proc CmdDelete_ { listb_name filelist_var frompwd topwd inst } {
           Log "Deleting $frompwd/[lindex $elem 1]"
           set r [regexp {ftp://([^/]*)(.*)} $frompwd/[lindex $elem 1] match ftpI file]
           if {$r} {
-            Try { FTP_Delete $ftpI "$file" } "" 1
             set glob(forceupdate) 1
+            Try { FTP_Delete $ftpI "$file" } "" 1
           }
         }
       }
@@ -373,8 +423,8 @@ proc CmdDelete_ { listb_name filelist_var frompwd topwd inst } {
         }
         if {$ask == 0} {
           Log "Deleting $frompwd/[lindex $elem 1]"
-          Try { DeleteFTPRecursive "$frompwd/[lindex $elem 1]" $inst } "" 1
           set glob(forceupdate) 1
+          Try { DeleteFTPRecursive "$frompwd/[lindex $elem 1]" $inst } "" 1
         }
       }
       default CantDoThat
@@ -413,15 +463,14 @@ proc CmdView_ { listb_name filelist_var frompwd topwd inst } {
               PopError "Can't parse $glob($inst,pwd) as ftp URL" 
             } else { 
               set r 0
-              if { ! [file exists /tmp/$env(USER)_fr_tmp] } {
-                set r [Try { exec $config(cmd,mkdir) /tmp/$env(USER)_fr_tmp } "" 1]
+              if { ! [file exists $glob(tmpdir)] } {
+                set r [Try { file mkdir $glob(tmpdir) } "" 1]
               }
               if { !$r } {
                 set size [lindex $fileelem 3]
                 if {[lindex $fileelem 2] == "fl"} {set size -1}
-                #catch { eval exec $config(cmd,rm) -f [glob /tmp/$env(USER)_fr_tmp/*] } out
-                set r [Try { FTP_GetFile $ftpI "$directory/[lindex $fileelem 1]" "/tmp/$env(USER)_fr_tmp/[lindex $fileelem 1]" $size 0 } "" 1]
-                if {$r == 0} { ViewAny /tmp/$env(USER)_fr_tmp/[lindex $fileelem 1]; set glob(havedoneftp) 1 }
+                set r [Try { FTP_GetFile $ftpI "$directory/[lindex $fileelem 1]" "$glob(tmpdir)/[lindex $fileelem 1]" $size 0 } "" 1]
+                if {$r == 0} { ViewAny $glob(tmpdir)/[lindex $fileelem 1]; set glob(havedoneftp) 1 }
               }
             }
           }
@@ -473,12 +522,12 @@ proc CmdCheckSize_ { listb_name filelist_var frompwd topwd inst } {
       d   -
       ld  -
       l   -
-      n   { lappend filenamelist $frompwd/[lindex $fileelem 1] }
+      n   { lappend filenamelist [Esc $frompwd/[lindex $fileelem 1]] }
       default CantDoThat
     }
   }
   if {$filenamelist != {}} {
-    set r [catch {eval exec $config(cmd,du) $filenamelist} out]
+    set r [catch {eval eval exec $config(cmd,du) $filenamelist} out]
     if {$r} {
       PopError $out
     } else {
@@ -526,14 +575,15 @@ proc CmdEdit_ { listb_name filelist_var frompwd topwd } {
     set elem [lindex $filelist $sel]
     switch [lindex $elem 2] {
       l   -
-      n   { lappend filenamelist "$frompwd/[lindex $elem 1]" }
+      n   { lappend filenamelist [Esc $frompwd/[lindex $elem 1]] }
       ld  -
       d   { PopInfo "Can't edit directory $frompwd/[lindex $elem 1] in the text editor" }
       default CantDoThat
     }
   }
   if {$filenamelist != {}} {
-    Try { eval eval exec [format $config(editor) $filenamelist] & } "Can't start editor" 1
+    # Three eval's in a row... New record? Yuck..
+    Try { eval eval eval exec [format $config(editor) $filenamelist] & } "Can't start editor" 1
   }
 }
 
@@ -591,7 +641,7 @@ proc CmdRename_ { listb_name filelist_var frompwd topwd } {
           if {$newname != ""} {
             if {[CheckWhoOwns $newname overwrite]} {
               Log "Renaming $frompwd/[lindex $elem 1] to $newname"
-              Try { exec $config(cmd,mv) "$frompwd/[lindex $elem 1]" "$newname" } "" 1
+              Try { file rename -force -- $frompwd/[lindex $elem 1] $newname } "" 1
             }
           }
         }
@@ -689,13 +739,20 @@ proc CmdArc_ { listb_name filelist_var frompwd topwd } {
     set elem [lindex $filelist $sel]
     switch [lindex $elem 2] {
       l   -
-      n   { Log "Packing $frompwd/[lindex $elem 1]"
-            Try { eval exec [format $config(cmd,pack) "$frompwd/[lindex $elem 1]"] } "" 1  $glob(async)
-          }
+      n   { 
+        Log "Packing $frompwd/[lindex $elem 1]"
+#        Try { eval exec [format $config(cmd,pack) {$frompwd/[lindex $elem 1]}] } "" 1  $glob(async)
+        Try { eval eval exec [format $config(cmd,pack) [Esc $frompwd/[lindex $elem 1]]] } "" 1  $glob(async)
+      }
       ld  -
-      d   { Log "Archiving $frompwd/[lindex $elem 1]"
-            Try { cd "$frompwd"; eval exec [format $config(cmd,arc) "[lindex $elem 1]" "[lindex $elem 1]"] } "" 1 $glob(async)
-          }
+      d   { 
+        Log "Archiving $frompwd/[lindex $elem 1]"
+        if {$config(cmd,archive) == "tar+gz {%s}"} {
+          Try { cd $frompwd; exec tar cf - [lindex $elem 1] | gzip > [lindex $elem 1].tar.gz } "" 1 $glob(async)
+        } else {
+          Try { cd $frompwd; eval eval exec [format $config(cmd,archive) [Esc [lindex $elem 1]] ] } "" 1 $glob(async)
+        }
+      }
       default CantDoThat
     }
   }
@@ -717,7 +774,7 @@ proc CmdPrint_ { listb_name filelist_var frompwd topwd } {
     switch [lindex $elem 2] {
       l   -
       n   { Log "Printing $frompwd/[lindex $elem 1]"
-            Try { eval exec [format $config(cmd,print) "$frompwd/[lindex $elem 1]"] & } "" 1 }
+            Try { eval eval exec [format $config(cmd,print) [Esc $frompwd/[lindex $elem 1]]] } "" 1 }
       ld  -
       d   { PopError "Can't print directories!!" }
       default CantDoThat
@@ -749,7 +806,7 @@ proc CmdMakeDir {} {
     Try { FTP_MkDir $ftpI "$dir" } "" 1
     set glob(forceupdate) 1
   } else {
-    Try { eval exec $config(cmd,mkdir,hier) "\"$newdir\"" } "" 1
+    Try { file mkdir $newdir } "" 1
   }
   UpdateWindow both
   set glob(forceupdate) 0
@@ -803,7 +860,7 @@ proc CmdCSelect_ { listb_name filelist_var frompwd topwd cmd } {
     switch [lindex $elem 2] {
       l   -
       n   { 
-            set r [catch { eval exec $cmd $frompwd/[lindex $elem 1] } outp]
+            set r [catch { eval exec $cmd {$frompwd/[lindex $elem 1]} } outp]
             if { $r != 0 } {
               $listb_name selection clear $sel
             }
@@ -891,17 +948,17 @@ proc CmdDiff_ { listb_name filelist_var frompwd topwd inst } {
   if {$null} return
 
   if {$file2 == ""} {
-    set file2 "$topwd/[lindex [lindex $glob([Opposite $inst],filelist) [$glob(win,[Opposite $inst]).frame_listb.listbox1 index active]] 1]"
-    set tmp "$file1"
-    set file1 "$file2"
-    set file2 "$tmp"
+    set file2 $topwd/[lindex [lindex $glob([Opposite $inst],filelist) [$glob(win,[Opposite $inst]).frame_listb.listbox1 index active]] 1]
+    set tmp $file1
+    set file1 $file2
+    set file2 $tmp
   }
 
-  set r [catch { eval exec [format $config(cmd,diff) "$file1" "$file2"] } outp]
+  set r [catch { eval eval exec [format $config(cmd,diff) [Esc $file1] [Esc $file2]] } outp]
   if { $r != 0 } {
     ViewString "Diffing $file1 and $file2" outp "diff.log"
   } else {
-    PopInfo "No difference between $file1 and $file2"
+    PopInfo "No difference between\n\n$file1\n\nand\n\n$file2"
   }
 }
 
@@ -950,7 +1007,7 @@ proc CopyFromFTPRecursive { fromdir todir inst resume } {
   set r [regexp {ftp://([^/]*)(.*)} $fromdir match ftpI directory]
   set dir [file tail $directory]
   Log "FTP recursive copy: Creating $todir/$dir"
-  exec $config(cmd,mkdir) "$todir/$dir"
+  file mkdir "$todir/$dir"
   Log "FTP recursive copy: Entering $fromdir"
   NewPwd $inst $fromdir
   UpdateWindow $inst
@@ -1034,7 +1091,7 @@ proc CmdFind { inst } {
     return
   }
   LogStatusOnly "Searching, please wait..."
-  set r [catch {exec $config(cmd,sh) -c "find . -name \"*${findname}*\" -print" 2> /dev/null } out]
+  set r [catch {exec $config(cmd,sh) -c "find . -name '*${findname}*' -print" 2> /dev/null } out]
   LogStatusOnly "Searching, please wait...done"
 #  if {$r} {
 #    PopError "Search error: $out"
@@ -1063,6 +1120,7 @@ proc CmdChmod_ { listb_name filelist_var frompwd topwd inst } {
   # For speed, (hopefully) we'll call by reference...
   global config glob
   upvar $filelist_var filelist
+  set fl {}
 
   foreach sel [$listb_name curselection] {
     if {[CheckAbort "Chmod"]} return
@@ -1071,15 +1129,63 @@ proc CmdChmod_ { listb_name filelist_var frompwd topwd inst } {
       l   -
       ld  -
       n   -
-      d   { 
-        set mode [lindex $elem 5]
-        set arg [ChmodDialog "$frompwd/[lindex $elem 1]" $mode]
-        if {$arg != ""} {
-          Try { eval exec $config(cmd,chmod) $arg "\"$frompwd/[lindex $elem 1]\"" } "" 1 $glob(async)
+      d   {
+        if {$fl == {}} {
+          set mode [lindex $elem 5]
         }
+        lappend fl [Esc $frompwd/[lindex $elem 1]]
       }
       default CantDoThat
     }
   }
+  if {$fl != ""} {
+    set arg [ChmodDialog "$frompwd/[lindex $elem 1]" $mode]
+    if {$arg != ""} {
+      Try { eval eval exec $config(cmd,chmod) $arg $fl } "" 1 $glob(async)
+    }
+  }
 }
 
+proc CmdGetHttp { inst } {
+  global glob config
+  
+  if {[IsFTP $glob($inst,pwd)]} {
+    PopInfo "You can only download HTTP files to a non-FTP directory"
+    return
+  }
+
+  set URL {}
+
+  while { 1 } {
+    set URL [EntryDialog "Get HTTP File" "Please enter HTTP URL to download" $URL]
+    if {$URL == ""} {
+      return
+    }
+    
+    if {[string range $URL 0 6] != "http://" } {
+      set URL "http://$URL"
+    }
+    
+    set r [regexp {http://([^/]*)(.*)} $URL match host_and_port location]
+    if {!$r} {
+      PopError "Could not parse $URL as an HTTP URL"
+      continue
+    } 
+    if {$location == ""} {
+      append URL /
+    }
+    break
+  }
+
+  set filename [file tail $URL]
+  if {[string range $URL end end] == "/"} {
+    append filename .html
+  }
+  set filename [EntryDialog "Get HTTP File" "Please edit filename to save to.\n(URL: $URL)" $filename]
+  if {$filename == ""} {
+    return
+  }
+  cd $glob($inst,pwd)
+  HTTP_Get $URL $filename
+  UpdateWindow both
+}
