@@ -16,7 +16,7 @@ proc AddConf { fid var_var setto expl } {
 
 proc SaveConfig {}  {
   global glob config env
-  set r [catch {open $env(HOME)/.fr w} fid]
+  set r [catch {open $glob(conf_dir)/config w} fid]
   if {$r != 0} { PopError "Saving config: $fid"; return "" }
   
   puts $fid {
@@ -106,15 +106,18 @@ proc SaveConfig {}  {
 
   AddConf $fid config(cmd,pack) "" {
 # The name of your favourite packer command. 
-# Used in the "Arc" button command to pack files. Run synchronously.
+# Used in the "Arc" button command to pack files. Run synchronously
+# or in the background.
 }
 
-  AddConf $fid config(cmd,arc) "" {
-# This is the name of your archiver program. The name of a directory
-# will show up twice as "%s". The archive created is supposed to contain
-# the contents of the directory.
-# Used in the "Arc" button command to pack/archive directories. Run 
-# synchronously.
+  AddConf $fid config(cmd,archive) "" {
+# This is the name of your archiver program. The archiver program is
+# passed the name of the directory selected and is executed in the
+# current directory. The archive created is supposed to contain the
+# contents of the directory. The default builtin "tar+gz {%s}" creates
+# an archive named <directory>.tar.gz packed with tar and gzip. This
+# command is used in the "Arc" button command to pack/archive
+# directories. Can be run synchronously or in the background.
 }
 
   puts $fid {
@@ -163,6 +166,19 @@ proc SaveConfig {}  {
   AddConf $fid config(ftp,useproxy) "" {
 # 1 -> Use defined ftp proxy
 # 0 -> Don't use proxy
+}
+
+  AddConf $fid config(ftp,cache,maxentries) "" {
+# Maximum number of directory listings the cache should hold.
+}
+
+  puts $fid {
+# ****** HTTP SETTINGS **************************************************
+}
+
+  AddConf $fid config(http,proxy) "" {
+# HTTP proxy. Format: host:port (for example: proxy.foo.com:80). 
+# Leave empty if you have a direct connection to the net.
 }
 
   puts $fid {
@@ -184,14 +200,24 @@ proc SaveConfig {}  {
 # You can edit this easier from the settings menu.
 }
 
-  AddConf $fid config(gui,color_select) "" {
-# Selection color.
+  AddConf $fid config(gui,color_select_bg) "" {
+# Selection background color.
+# You can edit this easier from the settings menu.
+}
+
+  AddConf $fid config(gui,color_select_fg) "" {
+# Selection foreground color.
 # You can edit this easier from the settings menu.
 }
 
   AddConf $fid config(gui,color_scheme) "" {
 # Scheme color.
-# Default Tk is to be "grey85"
+# Default Tk seems to be "grey85".
+# You can edit this easier from the settings menu.
+}
+
+  AddConf $fid config(gui,color_cursor) "" {
+# Cursor color.
 # You can edit this easier from the settings menu.
 }
 
@@ -323,6 +349,11 @@ proc SaveConfig {}  {
 # files. If 0, it will go ahead and delete without asking.
 }
 
+  AddConf $fid config(mwheel,delta) "" {
+# For "wheel"-mice (MS Intellimouse or Logitech MouseMan+), set
+# number of lines to scroll when turning mouse wheel.
+}
+
 
   puts $fid {
 # ****** DANGEROUS SETTINGS :-) *****************************************
@@ -344,14 +375,6 @@ proc SaveConfig {}  {
 # A few commands below do not have specified full path, that's because
 # they aren't very critical and seem to move around between different
 # versions of Unix.
-}
-
-  AddConf $fid config(cmd,mkdir) "" {
-# The make-dir command. This should almost always be /bin/mkdir.
-}
-
-  AddConf $fid config(cmd,mkdir,hier) "" {
-# The make-dir hierarchic command. This should almost always be /bin/mkdir -p.
 }
 
   AddConf $fid config(cmd,sh) "" {
@@ -360,10 +383,6 @@ proc SaveConfig {}  {
 
   AddConf $fid config(cmd,rm) "" {
 # The rm (remove-files) command. This should almost always be /bin/rm.
-}
-
-  AddConf $fid config(cmd,mv) "" {
-# The mv (move-files or rename) command. This should almost always be /bin/mv.
 }
 
   AddConf $fid config(cmd,cp) "" {
@@ -389,7 +408,7 @@ proc SaveConfig {}  {
 }
 
   close $fid
-  Log "Settings saved to $env(HOME)/.fr"
+  Log "Settings saved to $glob(conf_dir)/config"
 }
 
 proc InitConfig {} {
@@ -408,7 +427,7 @@ proc InitConfig {} {
   set config(cmd,term) "xterm"
   set config(cmd,diff) "diff -r {%s} {%s}"
   set config(cmd,pack) "gzip {%s}"
-  set config(cmd,arc) "tar cf - {%s} | gzip > {%s.tar.gz}"
+  set config(cmd,archive) "tar+gz {%s}"
 
   set config(cmd,unarc,extensions) {
   { {gzip -cd {%s} | tar xfBp -}
@@ -438,16 +457,20 @@ proc InitConfig {} {
   set config(ftp,anonymous) 1
   set config(ftp,useproxy) 1
   set config(ftp,login) {}
+  set config(ftp,cache,maxentries) 15
+  set config(http,proxy) {}
   set config(gui,color_bg) FloralWhite
   set config(gui,color_fg) Black
   set config(gui,color_cmd) "#ffffe4e679db"
-  set config(gui,color_select) "#f1a9ec43aa9d"
+  set config(gui,color_select_bg) "#f1a9ec43aa9d"
+  set config(gui,color_select_fg) black
   set config(gui,color_scheme) "#d1f0dfbebd10"
+  set config(gui,color_cursor) black
   set config(gui,font_scheme) ""
 
   foreach testfont { 
-      screen-bold-12 
       *-lucidatypewriter-bold-r-normal-*-12-*
+      screen-bold-12 
       *-lucidatypewriter-bold-r-normal-*-14-*
       *-courier-bold-r-normal-*-14-* 
       fixed 
@@ -500,19 +523,21 @@ proc InitConfig {} {
   { {gzip -cd {%s}}
     {*.gz}
     -viewtext }
+  { {rpm -qilp {%s}}
+    {*.rpm}
+    -viewtext }
 }
 
-  set config(cmd,mkdir) "/bin/mkdir"
-  set config(cmd,mkdir,hier) "/bin/mkdir -p"
   set config(cmd,sh) "/bin/sh"
   set config(cmd,rm) "/bin/rm"
-  set config(cmd,mv) "/bin/mv"
   set config(cmd,cp) "/bin/cp"
   set config(cmd,ln) "/bin/ln"
   set config(cmd,chmod) "/bin/chmod"
   set config(cmd,touch) "touch"
   set config(cmd,du) "du -s"
 
+  set config(mwheel,delta) 6
+
   set glob(searchstring) ""
   set glob(log) ""
   set glob(left,pwd)  [Pwd]
@@ -527,6 +552,12 @@ proc InitConfig {} {
   set glob(toplevelidx) 0
   set glob(entry_dialog_return) {}
   set glob(ftp_entry_dialog_return) {}
+  set glob(ftp,serverport,start) 5000
+  set glob(ftp,serverport,end) 65000
+  # This is an ugly hack to avoid reusing ports too fast, but it's because of a limitiation in the tcl socket
+  # command. (You can't make the socket -server command choose a port by itself...)
+  set glob(ftp,serverport,last) [expr 5000 + (([pid] * 100) % (65000 - 5000))]
+  set glob(ftp,serversocket,acceptlist) {}
   set glob(forceupdate) 0
   set glob(havedoneftp) 0
   set glob(enableautoupdate) 1
@@ -546,6 +577,8 @@ proc InitConfig {} {
   set glob(right,shell,history,flipping) 0
   set glob(left,shell,history,flipping,index) 0
   set glob(right,shell,history,flipping,index) 0
+  set glob(batchlist) {}
+  set glob(tmpdir) $glob(conf_dir)/tmp
 
   set glob(win,top) .fupper.ftop
   set glob(win,left) .fupper.fleft
@@ -563,10 +596,12 @@ proc InitConfig {} {
   set glob(gui,color_bg)      $config(gui,color_bg)
   set glob(gui,color_fg)      $config(gui,color_fg)
   set glob(gui,color_cmd)     $config(gui,color_cmd)
-  set glob(gui,color_select)  $config(gui,color_select)
+  set glob(gui,color_select_bg)  $config(gui,color_select_bg)
+  set glob(gui,color_select_fg)  $config(gui,color_select_fg)
   # These settings have not been set yet, must be done in ReConfigColors
   set glob(gui,color_scheme) ""
   set glob(gui,font_scheme) ""
+  set glob(gui,color_cursor) ""
 
   set glob(gui,font) $config(gui,font)
   set glob(gui,font_scheme) ""
@@ -598,8 +633,8 @@ proc InitConfig {} {
 
   set glob(history) ""
   # read history from disk
-  if { [file exists $env(HOME)/.fr_history ] } {
-    set r [catch {set fid [open $env(HOME)/.fr_history r];gets $fid glob(history);close $fid} out]
+  if { [file exists $glob(conf_dir)/history ] } {
+    set r [catch {set fid [open $glob(conf_dir)/history r];gets $fid glob(history);close $fid} out]
     if {$r} {
       puts "FileRunner: Can't read directory history from disk: $out"
     }
@@ -614,22 +649,23 @@ proc InitConfig {} {
 #  option add *Listbox.BorderWidth 2 widgetDefault # makes the spacing between list elements too large...
   option add *Frame.BorderWidth 2 widgetDefault
   option add *Entry.BorderWidth 2 widgetDefault
+#  option add *.Text.InsertBackground red widgetDefault
 #  option add *insertBorderWidth 2 widgetDefault
 #  option add *BorderWidth 1 widgetDefault
 }
 
 proc ReadConfig {} {
   global glob config env argv argv0
-  if {[file readable $glob(lib_fr)/.fr]} {
-    set r [catch {source $glob(lib_fr)/.fr} out]
+  if {[file readable $glob(lib_fr)/config]} {
+    set r [catch {source $glob(lib_fr)/config} out]
     if {$r} {
-      tk_dialog .error_dialog "Init error" "Reading system wide settings from $glob(lib_fr)/.fr:\n$out" error 0 "OK"
+      tk_dialog .error_dialog "Init error" "Reading system wide settings from $glob(lib_fr)/config:\n$out" error 0 "OK"
     }
   }
-  if {[file readable $env(HOME)/.fr]} {
-    set r [catch {source $env(HOME)/.fr} out]
+  if {[file readable $glob(conf_dir)/config]} {
+    set r [catch {source $glob(conf_dir)/config} out]
     if {$r} {
-      tk_dialog .error_dialog "Init error" "Reading user settings $env(HOME)/.fr:\n$out" error 0 "OK"
+      tk_dialog .error_dialog "Init error" "Reading user settings $glob(conf_dir)/config:\n$out" error 0 "OK"
     }
   }
   set glob(hotlist) $config(hotlist)
@@ -675,3 +711,52 @@ proc ConfigPwd {} {
   set glob(right,dirstack) $glob(right,pwd)
 }
 
+proc CheckConfigDir {} {
+  global env glob
+
+  if {[file isdirectory $glob(conf_dir)]} {
+    return
+  }
+  if {$glob(os) != "WIN32" && [file exists $env(HOME)/.fr]} {
+    set button [tk_dialog .questDialog "Question" \
+        "The configuration files have moved for this version of FileRunner. Is it OK to move the config files to \
+the new directory $env(HOME)/.fr? (The old .fr file will be called .fr/config, the .fr_cmds file will be called .fr/cmds \
+and the .fr_history file will be called .fr/history)" \
+        question 0 "Yes" "No(Exit)"]
+    if {$button == 1} {
+      exit 0
+    }
+    set cmds { 
+      "file rename $env(HOME)/.fr $env(HOME)/.fr_tmp" 
+      "file mkdir $env(HOME)/.fr"
+      "file rename $env(HOME)/.fr_tmp $env(HOME)/.fr/config" 
+    }
+    foreach cmd $cmds {
+      set r [catch $cmd out]
+      if {$r} {
+        PopErrorSimple "$cmd : $out"
+        exit 1
+      }
+    }
+    set cmds { 
+      "file rename $env(HOME)/.fr_cmds $env(HOME)/.fr/cmds" 
+      "file rename $env(HOME)/.fr_history $env(HOME)/.fr/history" 
+      "file rename $env(HOME)/.fr_ver $env(HOME)/.fr/version" }
+    foreach cmd $cmds {
+      set r [catch $cmd out]
+    }
+    return
+  }
+  set button [tk_dialog .questDialog "Question" \
+      "The FileRunner configuration directory $glob(conf_dir) does not exist. Ok to create it?" \
+      error 0 "Yes" "No"]
+  if {$button == 1} {
+    return
+  }
+  set r [catch "file mkdir $glob(conf_dir)" out]
+  if {$r} {
+    PopErrorSimple "$out"
+    exit 1
+  }
+}
+
