@@ -4,20 +4,25 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <sys/stat.h>
+#include <sys/types.h>
+#include <errno.h>
+#include <fcntl.h>
+#include <string.h>
+#include <limits.h>
+
+#ifdef WIN32
+#include <winsock.h>
+#else
 #include <unistd.h>
 #include <pwd.h>
 #include <grp.h>
-#include <sys/types.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <netdb.h>
-#include <errno.h>
 #include <sys/utsname.h>
-#include <fcntl.h>
-#include <string.h>
 #include <arpa/inet.h>
 #include <sys/time.h>
-#include <limits.h>
+#endif
 
 #ifdef IRIX
 #include <bstring.h>
@@ -53,8 +58,6 @@ static int GetStringFromMode(ClientData clientData, Tcl_Interp* interp,
                            int argc, char* argv[]);
 static int GetUidGidString(ClientData clientData, Tcl_Interp* interp, 
                            int argc, char* argv[]);
-static int GetUsecTime(ClientData clientData, Tcl_Interp* interp, 
-                           int argc, char* argv[]);
 static int FTP_CreateServerSocket(ClientData clientData, Tcl_Interp* interp, 
                            int argc, char* argv[]);
 static int FTP_AcceptConnect(ClientData clientData, Tcl_Interp* interp, 
@@ -71,17 +74,20 @@ static int FTP_ReadText(ClientData clientData, Tcl_Interp* interp,
                            int argc, char* argv[]);
 static int FTP_WriteText(ClientData clientData, Tcl_Interp* interp, 
                            int argc, char* argv[]);
-static int CheckOwner(ClientData clientData, Tcl_Interp* interp, 
-                           int argc, char* argv[]);
 static int GetDF(ClientData clientData, Tcl_Interp* interp, 
                            int argc, char* argv[]);
 static int ClockMilliSeconds(ClientData clientData, Tcl_Interp* interp, 
                            int argc, char* argv[]);
+static int GetEuid(ClientData clientData, Tcl_Interp* interp, 
+                           int argc, char* argv[]);
 
 static long int  ToNum(char* string);
 
 static int dateformat;
 
+#ifdef WIN32
+_declspec(dllexport)
+#endif
 int
 Ext_Init(interp)
     Tcl_Interp *interp;		/* Interpreter for application. */
@@ -90,7 +96,6 @@ Ext_Init(interp)
     Tcl_CreateCommand(interp, "GetTimeFromSecsSetFormat", GetTimeFromSecsSetFormat, NULL, NULL);
     Tcl_CreateCommand(interp, "GetStringFromMode", GetStringFromMode, NULL, NULL);
     Tcl_CreateCommand(interp, "GetUidGidString", GetUidGidString, NULL, NULL);
-    Tcl_CreateCommand(interp, "GetUsecTime", GetUsecTime, NULL, NULL);
     Tcl_CreateCommand(interp, "FTP_CreateServerSocket", FTP_CreateServerSocket, NULL, NULL);
     Tcl_CreateCommand(interp, "FTP_Socket", FTP_Socket, NULL, NULL);
     Tcl_CreateCommand(interp, "FTP_AcceptConnect", FTP_AcceptConnect, NULL, NULL);
@@ -99,9 +104,9 @@ Ext_Init(interp)
     Tcl_CreateCommand(interp, "FTP_OpenFile", FTP_OpenFile, NULL, NULL);
     Tcl_CreateCommand(interp, "FTP_ReadText", FTP_ReadText, NULL, NULL);
     Tcl_CreateCommand(interp, "FTP_WriteText", FTP_WriteText, NULL, NULL);
-    Tcl_CreateCommand(interp, "CheckOwner", CheckOwner, NULL, NULL);
     Tcl_CreateCommand(interp, "GetDF", GetDF, NULL, NULL);
     Tcl_CreateCommand(interp, "ClockMilliSeconds", ClockMilliSeconds, NULL, NULL);
+    Tcl_CreateCommand(interp, "GetEuid", GetEuid, NULL, NULL);
     return TCL_OK;
 }
 
@@ -150,6 +155,9 @@ GetStringFromMode(ClientData clientData, Tcl_Interp* interp,
   mode = strtol(argv[1], &p, 0);
   HANDLE2(*p != '\0', "Error converting arg to int");
 
+#ifdef WIN32
+  sprintf(interp->result, "0x%x", mode);
+#else
   sprintf(interp->result, "%c%c%c%c%c%c%c%c%c", 
     mode & S_IRUSR ? 'r' : '-',
     mode & S_IWUSR ? 'w' : '-',
@@ -160,6 +168,7 @@ GetStringFromMode(ClientData clientData, Tcl_Interp* interp,
     mode & S_IROTH ? 'r' : '-',
     mode & S_IWOTH ? 'w' : '-',
     mode & S_ISVTX ? 'T' : (mode & S_IXOTH ? 'x' : '-'));
+#endif
   return TCL_OK;
 }
 
@@ -178,6 +187,9 @@ GetUidGidString(ClientData clientData, Tcl_Interp* interp,
 
   HANDLE2(argc != 3, "Wrong # of args");
 
+#ifdef WIN32
+  sprintf(interp->result, "");
+#else
   uid = strtol(argv[1], &p, 0);
   HANDLE2(*p != '\0', "Error converting arg to int");
 
@@ -202,6 +214,7 @@ GetUidGidString(ClientData clientData, Tcl_Interp* interp,
   }
 
   sprintf(interp->result, "%s/%s",uidstring,gidstring);
+#endif
   return TCL_OK;
 }
 
@@ -257,7 +270,7 @@ FTP_CreateServerSocket(ClientData clientData, Tcl_Interp* interp,
   return TCL_OK;
 }
 
-/* FTP_Socket host port -> return {local-host socket-fd}*/
+/* FTP_Socket host port -> return {socket-fd local-host}*/
 static int 
 FTP_Socket(ClientData clientData, Tcl_Interp* interp, 
                            int argc, char* argv[])
@@ -507,6 +520,7 @@ FTP_Copy(ClientData clientData, Tcl_Interp* interp,
   return TCL_OK;
 }
 
+
 static long int 
 ToNum(char* string) {
   char *p;
@@ -516,51 +530,15 @@ ToNum(char* string) {
   return i;
 }
 
-/* not implemented completely */
-static int 
-GetUsecTime(ClientData clientData, Tcl_Interp* interp, 
-                           int argc, char* argv[])
-{
-#if 0
-  unsigned long int i;
-
-  HANDLE2(argc != 0, "Wrong # of args");
-  sprintf(interp->result, "%lu", i);
-#endif
-  return TCL_ERROR;
-}
-
-
-/* CheckOwner filename -> return 1 if the user owns it, 0 otherwise */
-static int 
-CheckOwner(ClientData clientData, Tcl_Interp* interp, 
-                           int argc, char* argv[])
-{
-  int i;
-  uid_t uid;
-  struct stat b;
-  HANDLE2(argc != 2, "Wrong # of args");
-
-  i = lstat(argv[1], &b);
-  if (i) 
-  {
-    strcpy(interp->result, "1");
-    return TCL_OK;
-  }
-  uid = geteuid();
-  if (b.st_uid == uid)
-    strcpy(interp->result, "1");
-  else
-    strcpy(interp->result, "0");
-
-  return TCL_OK;
-}
-
 /* ClockMilliSeconds -> return milliseconds since startup */
 static int 
 ClockMilliSeconds(ClientData clientData, Tcl_Interp* interp, 
                            int argc, char* argv[])
 {
+#ifdef WIN32
+  sprintf(interp->result, "%f", 0.0);
+  return TCL_OK;
+#else
   struct timeval date;
   struct timezone tz;
   double t;
@@ -573,6 +551,22 @@ ClockMilliSeconds(ClientData clientData, Tcl_Interp* interp,
 
   sprintf(interp->result, "%f", t);
   return TCL_OK;
+#endif
+}
+
+/* GetEuid -> return effective user ID */
+static int 
+GetEuid(ClientData clientData, Tcl_Interp* interp, 
+                           int argc, char* argv[])
+{
+  HANDLE2(argc != 1, "Wrong # of args");
+
+#ifdef WIN32
+  sprintf(interp->result, "%d", 1); /* Anything not zero... */
+#else
+  sprintf(interp->result, "%d", geteuid());
+#endif
+  return TCL_OK;
 }
 
 
@@ -624,6 +618,25 @@ GetDF(ClientData clientData, Tcl_Interp* interp,
 
 #else
 
+#ifdef WIN32
+
+/* GetDF path -> returns free bytes in MB in path */
+static int 
+GetDF(ClientData clientData, Tcl_Interp* interp, 
+                           int argc, char* argv[])
+{
+  int i;
+  double b, x;
+  char tmp[5];
+  HANDLE2(argc != 2, "Wrong # of args");
+
+  sprintf(interp->result, "?MB");
+
+  return TCL_OK;
+}
+
+#else
+
 #ifdef OSF
 #  include <sys/mount.h>
 #else
@@ -685,3 +698,5 @@ GetDF(ClientData clientData, Tcl_Interp* interp,
 }
 
 #endif
+
+#endif
